#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var commander_1 = __importDefault(require("commander"));
var _a = require("../package.json"), version = _a.version, bin = _a.bin;
var name = Object.keys(bin)[0];
var ValidationService_1 = require("./ValidationService");
__exportStar(require("./ValidationService"), exports);
var NAME_DELIMITER = ":";
function parseRecompilationTarget(input) {
    var delimiterIndex = input.indexOf(NAME_DELIMITER);
    if (delimiterIndex === -1) {
        return input;
    }
    return { path: input.slice(0, delimiterIndex), name: input.slice(delimiterIndex + 1) };
}
function matches(contract, recompilationTarget) {
    if (!recompilationTarget) {
        return false;
    }
    if (typeof recompilationTarget === "string") {
        if (recompilationTarget === contract.name || recompilationTarget === contract.compiledPath) {
            return true;
        }
        return false;
    }
    var nameMatches = recompilationTarget.name === contract.name;
    var pathMatches = recompilationTarget.path === contract.compiledPath;
    return nameMatches && pathMatches;
}
function logOutput(recompilableContracts, programOptions) {
    switch (recompilableContracts.length) {
        case 0: {
            console.log("Could not find the " + OPTION_NAME + " target: " + programOptions.prepareJson);
            process.exitCode = 1;
            break;
        }
        case 1: {
            var standardJson = recompilableContracts[0].getStandardJson(programOptions.settings);
            var prettifiedStandardJson = JSON.stringify(standardJson, null, programOptions.pretty ? 2 : undefined);
            console.log(prettifiedStandardJson);
            break;
        }
        default: {
            console.log("Ambiguous " + OPTION_NAME + " target: " + programOptions.prepareJson);
            console.log(recompilableContracts.length + " instances encountered.");
            process.exitCode = 1;
            break;
        }
    }
}
var OPTION_NAME = "prepare-json";
commander_1.default
    .name(name)
    .helpOption("-h, --help", "Output the help message.")
    .version(version, "-v, --version", "Output the current version.")
    .description("A tool that validates smart contracts contained in the provided files, directories and zip archives.")
    .usage("[options] <path>...")
    .option("-j, --" + OPTION_NAME + " <path:name>", "Output only a JSON object to be used as standard-json input to solc.\n" +
    "The expected argument ought to be of the form path:name; ommiting any of the two is tolerated if unambiguous.\n" +
    ("E.g. `" + name + " --" + OPTION_NAME + " foo/bar/Contract.sol:ContractName path/to/dir path/to/zip`\n"))
    .option("-S, --no-settings", "Omit the original settings when preparing a standard-json.\n" +
    ("Only available with the --" + OPTION_NAME + " option\n"), true)
    .option("-p, --pretty", "Output the standard json in a pretty format.\n" +
    ("Only available with the --" + OPTION_NAME + " option\n"));
if (require.main === module) {
    commander_1.default.parse(process.argv);
    var fileNames = commander_1.default.args; // what's left from process.argv
    var fileNamesJoint_1 = fileNames.join(" ");
    if (!fileNames.length) {
        commander_1.default.help();
    }
    var recompilationTarget_1 = commander_1.default.prepareJson ? parseRecompilationTarget(commander_1.default.prepareJson) : commander_1.default.prepareJson;
    var validationService = new ValidationService_1.ValidationService();
    var checkedContracts = [];
    var ignoring = [];
    try {
        checkedContracts = validationService.checkPaths(fileNames, ignoring);
    }
    catch (err) {
        console.log(err.message);
        process.exitCode = 1;
    }
    var recompilableContracts_1 = [];
    var contractMessages_1 = [];
    var invalidContracts_1 = 0;
    checkedContracts.forEach(function (contract) {
        if (matches(contract, recompilationTarget_1)) {
            if (contract.isValid()) {
                recompilableContracts_1.push(contract);
            }
            else {
                console.log(contract.info);
                console.log("The " + OPTION_NAME + " target is not a valid contract! Exiting.");
                process.exit(1);
            }
        }
        else if (recompilationTarget_1 === undefined) {
            var msg = contract.info; // should be without a trailing newline
            if (contract.isValid()) {
                msg += "\n";
                msg += "  To recompile, use: `" + name + " --" + OPTION_NAME + " " + contract.compiledPath + ":" + contract.name + " " + fileNamesJoint_1;
                msg += " | solc --standard-json`";
            }
            else {
                invalidContracts_1++;
            }
            contractMessages_1.push(msg);
        }
    });
    if (contractMessages_1.length) {
        console.log(contractMessages_1.join("\n\n"));
    }
    if (recompilationTarget_1 !== undefined) {
        logOutput(recompilableContracts_1, commander_1.default.opts());
    }
    else if (ignoring.length) {
        console.log("\nFiles ignored due to readability issues:");
        ignoring.forEach(function (ignored) { return console.log("  " + ignored); });
    }
    if (invalidContracts_1 > 0) {
        process.exitCode = 1;
    }
}
//# sourceMappingURL=index.js.map